#include <avr/pgmspace.h>
#include <TFT.h>
#include <Wire.h>
#include "RTC.h"
#include "Touch.h"
#include "fastTFT.h"
#include "TriangleMesh.h"
#include "SimpleDLS.h"
#include "Clock.h"

// ==================================================================
// ==================================================================
//  HSVClock
//    Mark Wilson 2014
//  Colourful clock using a Seeed TFT to display the time as brightly coloured rectangular/triangular cells
//  Includes 
//   * fast TFT update
//   * simplified button-like touch screen 
//   * daylight savings time adjustment
//   * startup and configure "splash" screens
//   * easter egg (recipient's birthday)
// ==================================================================
//  Released under CC BY-NC 3.0, 
//  http://creativecommons.org/licenses/by-nc/3.0/
//  (kiwimew at gmail dot com)
//  -----------------------------------------------------------------
//  November 2014: Initial version
// ==================================================================

TriangleMesh mesh;
Clock clock(mesh);

void setup()
{
#ifdef CLOCK_DEBUG  
  Serial.begin(38400);
#endif  
  fastTFT::Init(false, 0);  // 0m delay works for me
  fastTFT::BlackPixels(76800L);
  rtc.Init();
  touch.Init(A3, A1, A2, A0, 0x80);
  clock.Init();
}


void loop()
{
  clock.Loop();
}
