#include "arduino.h"
#include "Touch.h"

// based on demo sketches.


// debounce
#define HOLD_TIME_MS 50

void Touch::Init(int pinXpos, int pinXneg, int pinYpos, int pinYneg, int ThresholdPressure)
{
  m_PinXpos = pinXpos;
  m_PinXneg = pinXneg;
  m_PinYpos = pinYpos;
  m_PinYneg = pinYneg;
  
  m_iThresholdPressure = ThresholdPressure;
  
  m_iPrevReading = 0;
  m_iPrevState = -1;
  m_iTransitionTimeMS = millis();
}

bool Touch::Pressed()
{
  // debounced "touch button", true if pressed, transitions only (no repeat of a press)
  int ThisReading = (Pressure() >= m_iThresholdPressure)?1:0;
  
  if (ThisReading != m_iPrevReading)
  {
    // state change, reset the timer
    m_iPrevReading = ThisReading;
    m_iTransitionTimeMS = millis();
  }
  else if (ThisReading != m_iPrevState &&
           (millis() - m_iTransitionTimeMS) >= HOLD_TIME_MS)
  {
    // a state other than the last one and held for long enough
    m_iPrevState = ThisReading;
    return (ThisReading == 1);
  }
  return false;  
}

int Touch::Col() 
{
   // X in LANDSCAPE mode
   pinMode(m_PinXpos, INPUT_PULLUP);
   pinMode(m_PinXneg, INPUT_PULLUP);
   
   pinMode(m_PinYpos, OUTPUT);
   digitalWrite(m_PinYpos, HIGH);
   pinMode(m_PinYneg, OUTPUT);
   digitalWrite(m_PinYneg, LOW);
   
   // convert to pixels assuming a ~100-step margin at each side
   return map(1023 - analogRead(m_PinXneg), 100, 900, 0, 320);
}

int Touch::Row() 
{
   // Y in LANDSCAPE mode
   pinMode(m_PinYpos, INPUT_PULLUP);
   pinMode(m_PinYneg, INPUT_PULLUP);
   
   pinMode(m_PinXpos, OUTPUT);
   digitalWrite(m_PinXpos, HIGH);
   pinMode(m_PinXneg, OUTPUT);
   digitalWrite(m_PinXneg, LOW);
   
   // convert to pixels assuming a ~100-step margin at each side
   return map(1023 - analogRead(m_PinYpos), 100, 900, 0, 240);
}

word Touch::TouchedCell()
{
  // if a touch is detected returns a single bit representing a cell on a 4x4 grid
  // thus bit0 is a press on the bottom-left cell
  // bit3 is a press on the bottom right cell
  // returns 0 if no touch
    
  if (Pressed())
  {
    int row = Row()/60;
    int col = Col()/80;
   
    return 1 << (col + row*4);
  }
  return 0;
}

int Touch::Pressure()
{
  // Set X+ to ground
  pinMode(m_PinXpos, OUTPUT);
  digitalWrite(m_PinXpos, LOW);
  
  // Set Y- to VCC
  pinMode(m_PinYneg, OUTPUT);
  digitalWrite(m_PinYneg, HIGH); 
  
  // Hi-Z X- and Y+
  pinMode(m_PinXneg, INPUT_PULLUP);
  pinMode(m_PinYpos, INPUT_PULLUP);
  
  return (1023 - (analogRead(m_PinYpos)-analogRead(m_PinXneg)));
}

Touch touch;
