#ifndef _TRIANGLEMESH_H
#define _TRIANGLEMESH_H

#include <arduino.h>
typedef word Colour;

// byte or word colours: words are slightly slower but give better colours
//#define BYTE_HSV

#ifdef BYTE_HSV
typedef byte ColourComponent;
#else
typedef word ColourComponent;
#endif

#define SetXY SetXYLandscape // (vs SetXYPortrait)

class TriangleMesh
{
public:
  TriangleMesh();
  virtual ~TriangleMesh();
#ifdef _WINDOWS
  void Paint(CDC& dc);
#else
  void Paint();
#endif

  void Clear();
  int  Display(const char* Str, int row, int col);
  void Display(prog_uint32_t* Defns, int Count, int row, int col);
  void RandomizeColour(bool smear);
  void RandomizeMesh(bool distort);
  bool RandomizeNode(int x, int y);

private:
  virtual bool DimDisplay();
  void PaintCell(int col, int row, char* aX, char* bX, char* cX);
  Colour HSVtoRGB(ColourComponent h, ColourComponent s, ColourComponent v);
  void GetCellColours(int col, int row, bool on, bool diagonal, Colour& rgb1, Colour& rgb2);
  virtual unsigned long GetCharDefinition(char Char);
  void SetChar(unsigned long charDef, int baseRow, int& baseCol);
  void RandomizeCell(byte& cell);
  int  CellOffset(byte nibble, int max);
  void GetVertex(int x0, int y0, int col, int row, int& x, int& y);
  void ClearCellStates();
  void Bresenham(int x0,int y0,int x1,int y1, char* min, char* max);

  static const prog_uint32_t charDefs[] PROGMEM;

  // defines to save memory:
  #define fontWidth       (4)
  #define fontHeight      (5)
  #define screenWidth     (320)
  #define screenHeight    (240)
  #define cellWidth       (screenWidth/(fontWidth*4 + 1))
  #define cellHeight      (cellWidth)
  #define cellsAcross     (screenWidth/cellWidth)
  #define cellsDown       (screenHeight/cellHeight)
  #define edgeWidth       ((screenWidth - cellsAcross*cellWidth)/2)
  #define edgeHeight      ((screenHeight - cellsDown*cellHeight)/2)
  #define maxVertexOffset (4)
  #define maxVertexY      (cellHeight + 2*maxVertexOffset + 1)
  #define vertexNULL      (-99)
#ifdef BYTE_HSV
  #define maxDeltaV       ((ColourComponent)(51)) // 20% of 255
  #define maxDeltaH       ((ColourComponent)(13)) // 5% of 255
#else
  #define maxDeltaV       ((ColourComponent)(13107)) // 20% of 65535
  #define maxDeltaH       ((ColourComponent)(3276)) // 5% of 65535
#endif
  #define cellStateBit        (0x08)  // cell is on
  #define cellDiagonalBit     (0x80)  // cell diagonal is '/'
  #define distortVerticesFlag (0x01)  // shift the vertices
  #define smearColoursFlag    (0x02)  // change the colour
  #define newMeshFlag         (0x04)  // vertices in new locations

  byte flags;
  ColourComponent cellColourH;
  ColourComponent cellColourS;
  ColourComponent cellColourV;
  int cellDeltaHCol;
  int cellDeltaHRow;

  // DXXXSYYY dX/DY+/-1..<maxVertexOffset>, D is diagonal, S is on/off
  byte cells[cellsAcross + 1][cellsDown + 1];
};
#endif
