#include <avr/pgmspace.h>
#include "fastTFT.h"

byte fastTFT::portDHi = 0;
byte fastTFT::portBHi = 0;
byte fastTFT::portBHiWrite = 0;
byte fastTFT::portDLo = 0;
byte fastTFT::portBLo = 0;
byte fastTFT::portBLoWrite = 0;

byte fastTFT::altPortDHi = 0;
byte fastTFT::altPortBHi = 0;
byte fastTFT::altPortBHiWrite = 0;
byte fastTFT::altPortDLo = 0;
byte fastTFT::altPortBLo = 0;
byte fastTFT::altPortBLoWrite = 0;

// Initialisation sequence, taken from TFT library
// Entries in pairs:
//   Cmd, Data,
// to send or
//   0xFFFF, delay divisor,
// to pause or
//   0xFFFF, 0xFFFF,
// to end
// Command name annotations from Table 9 "System Function Command List" in ST7781R_datasheet.pdf
#define WAIT 0xFFFF
prog_uint16_t fastTFT::initData[] PROGMEM =
{
 0x01, 0x0100,  // Driver Output Control
 0x02, 0x0700,  // LCD Driving Wave Control (?)
 0x03, 0x1030,  // Entry Mode [BGR=1, portrait]
 0x04, 0x0000,  // Resize Control
 0x08, 0x0302,  // Display control 2
 0x0A, 0x0000,  // Display Control 4
 0x0C, 0x0000,  // RGB Display Interface Control 1
 0x0D, 0x0000,  // Frame Marker Position
 0x0F, 0x0000,  // RGB Display Interface Control 2
 WAIT, 1,       // (delayMS/1)
 0x30, 0x0000,  // Gamma Control 1
 0x31, 0x0405,  // Gamma Control 2
 0x32, 0x0203,  // Gamma Control 3
 0x35, 0x0004,  // Gamma Control 4
 0x36, 0x0B07,  // Gamma Control 5
 0x37, 0x0000,  // Gamma Control 6
 0x38, 0x0405,  // Gamma Control 7
 0x39, 0x0203,  // Gamma Control 8
 0x3C, 0x0004,  // Gamma Control 9
 0x3D, 0x0B07,  // Gamma Control 10
 0x20, 0x0000,  // Horizontal DRAM Address Set
 0x21, 0x0000,  // Vertical DRAM Address Set
 0x50, 0x0000,  // Horizontal Address Start Position
 0x51, 0x00EF,  // Horizontal Address End Position 
 0x52, 0x0000,  // Vertical Address Start Position
 0x53, 0x013F,  // Vertical Address End Position 
 WAIT, 1,       // (delayMS/1)
 0x60, 0xA700,  // Driver Output Control 2
 0x61, 0x0001,  // Base Image Display Control
 0x90, 0x003A,  // Panel Interface Control 1
 0x95, 0x021E,  // Panel Interface Control 2
 0x80, 0x0000,  // Partial Image 1 Display Position
 0x81, 0x0000,  // Partial Image 1 Area (Start Line)
 0x82, 0x0000,  // Partial Image 1 Area (End Line)
 0x83, 0x0000,  // Partial Image 2 Display Position
 0x84, 0x0000,  // Partial Image 2 Area (Start Line)
 0x85, 0x0000,  // Partial Image 2 Area (End Line)
 0xFF, 0x0001,  // NVM Command Enable
 0xB0, 0x140D,  // Power Control 2
 0xFF, 0x0000,  // NVM Command Enable
 WAIT, 1,       // (delayMS/1)
 0x07, 0x0133,  // Display Control 1
 WAIT, 2,       // (delayMS/2)
 0x10, 0x14E0,  // Power Control 1
 WAIT, 1,       // (delayMS/1)
 0x07, 0x0133,  // Display Control 1

 0xFFFF, 0xFFFF
};

word cmd03;

void fastTFT::Init(bool portrait, int delayMS)
{
  CS_OUTPUT;
  RD_OUTPUT;
  WR_OUTPUT;
  RS_OUTPUT;

  Tft.all_pin_output();
  Tft.all_pin_low();
  delay(delayMS);
  
  word cmd;
  word data;
  prog_uint16_t* pPtr = initData;
  do
  {
    cmd  = pgm_read_word(pPtr++);
    data = pgm_read_word(pPtr++);
    if (cmd == 0xFFFF)
    {
      if (data == 0xFFFF)
      {
        break;  // end of the table
      }
      else
      {
        delay(delayMS/data);
      }
    }
    else
    {
      FAST_TFT_COMMAND(cmd);
      SendData(data);
    }
  } while (true);
  
  if (!portrait)
  {
    // landscape mode
    FAST_TFT_COMMAND(0x03);
    SendData(0x1098);  // keep TFT's BRG setting, ORG=1, orientation=0x18
  }
  // cursor was set to 0,0 above, get ready to write
  FAST_TFT_COMMAND(0x22);
}

